/*
Copyright (c) 2003 Hartmut Henkel, <hartmut_henkel@gmx.de>

This file is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by Free
Software Foundation; either version 2 of the License, or (at your option)
any later version.

This file is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this file; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

$Id: avlstuff.c,v 1.20 2003/10/03 19:08:54 hahe Exp hahe $

formatted by indent -kr
*/

#include "ptexlib.h"
#include <kpathsea/c-vararg.h>
#include <kpathsea/c-proto.h>
#include "avl.h"


typedef struct PdfObj_ {
    int objptr;
    int info;
} PdfObj;

/* One AVL tree for each obj_type 0...9 */

static struct avl_table *PdfObjTree[10] =
    { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };


/* memory management functions for avl */

void *avl_xmalloc(struct libavl_allocator *allocator, size_t size)
{
    assert(allocator != NULL && size > 0);
    return xmalloc(size);
}

void avl_xfree(struct libavl_allocator *allocator, void *block)
{
    assert(allocator != NULL && block != NULL);
    xfree(block);
}

struct libavl_allocator avl_xallocator = {
    avl_xmalloc,
    avl_xfree
};


/* comparison function used by avl */

int compare_info(const void *pa, const void *pb, void *param)
{
    const int *a, *b;
    int as, ae, bs, be, al, bl, i;

    a = &(((const PdfObj *) pa)->info);
    b = &(((const PdfObj *) pb)->info);
    if ((*a < 0) && (*b < 0)) {	/* string comparison */
	as = strstart[-(*a)];
	ae = strstart[-(*a) + 1];	/* start of next string in pool */
	bs = strstart[-(*b)];
	be = strstart[-(*b) + 1];
	al = ae - as;
	bl = be - bs;
	if (al < bl)		/* compare first by string length */
	    return -1;
	else if (al > bl)
	    return +1;
	else
	    for (i = 0; i < al; i++, as++, bs++) {
		if (strpool[as] < strpool[bs])
		    return -1;
		if (strpool[as] > strpool[bs])
		    return +1;
	    }
	return 0;
    } else {			/* integer comparison */
	if (*a < *b)
	    return -1;
	else if (*a > *b)
	    return +1;
	else
	    return 0;
    }
}


void avlputobj(integer objptr, integer t)
{
    static void *p;
    static void **pp;
    static PdfObj *tmp;

    if (PdfObjTree[t] == NULL) {
	PdfObjTree[t] = avl_create(compare_info, NULL, &avl_xallocator);
	if (PdfObjTree[t] == NULL)
	    pdftex_fail("avlstuff.c: avl_create() PdfObjTree failed");
    }
    tmp = xmalloc(sizeof(PdfObj));
    tmp->objptr = objptr;
    tmp->info = objinfo(objptr);
    p = avl_find(PdfObjTree[t], tmp);
    if (p == NULL) {
	pp = avl_probe(PdfObjTree[t], tmp);
	if (pp == NULL)
	    pdftex_fail
		("avlstuff.c: avl_probe() out of memory in insertion");
    }
}


/* replacement for linear search pascal function "find_obj" */

integer avlfindobj(integer t, integer i, integer byname)
{
    static PdfObj *p;
    static PdfObj tmp;

    tmp.objptr = 0;
    if (byname > 0)
	tmp.info = -i;
    else
	tmp.info = i;
    if (PdfObjTree[t] == NULL)
	return 0;
    p = (PdfObj *) avl_find(PdfObjTree[t], &tmp);
    if (p == NULL)
	return 0;
    else
	return p->objptr;
}

/* end of file avlstuff.c */
